@@ -174,17 +174,17 @@ class PipS3:
         # The files does not exist we can upload
         except self.s3_client.exceptions.ClientError:
 
-            ExtraArgs = {}
+            extra_args = {}
             if public:
-                ExtraArgs["ACL"] = "public-read"
+                extra_args["ACL"] = "public-read"
 
             if owner_full_control:
-                ExtraArgs["GrantFullControl"] = "full-control"
+                extra_args["ACL"] = "bucket-owner-full-control"
 
             self.s3_client.upload_file(pkg_path,
                                        self.bucket,
                                        key,
-                                       ExtraArgs=ExtraArgs)
+                                       ExtraArgs=extra_args)
             return
 
         raise PackageExistsException(
@@ -210,20 +210,35 @@ class PipS3:
         key = f'{self.prefix}/{package_name}/index.html'
         logger.info("Uploading index to s3://%s/%s", self.bucket, key)
 
-        put_object = functools.partial(self.s3_client.put_object,
-                                       Bucket=self.bucket,
-                                       Key=key,
-                                       Body=generated_index.encode('utf-8'),
-                                       ContentType="text/html")
-
+        acl = 'bucket-owner-full-control' if owner_full_control else ''
         if public:
-            put_object = functools.partial(put_object, ACL='public-read')
+            acl = 'public-read'
 
-        if owner_full_control:
-            put_object = functools.partial(put_object,
-                                           GrantFullControl='full-control')
+        self.s3_client.put_object(Bucket=self.bucket,
+                                  Key=key,
+                                  Body=generated_index.encode('utf-8'),
+                                  ACL=acl,
+                                  ContentType="text/html")
 
-        put_object()
+
+def get_package_name(upload_file: str) -> str:
+    """determine the package name from the artifacts to be uploaded. According to pypi,
+    if upload_file is a tar.gz, then the package name is already canonical.
+    if upload_file is a whl, then we need to replace _ with -.
+    See tests for examples.
+
+    Args:
+        upload_file (str): path to the artifacts, tar.gz or whl
+
+    Returns:
+        str: package name
+    """
+    stem = os.path.basename(upload_file)
+    splitted = stem.split("-")
+    if upload_file.endswith(".gz"):
+        return "-".join(splitted[:-1]) # the last one is version
+    # else is "whl"
+    return splitted[0].replace('_', '-')
 
 
 def publish_packages(endpoint: str,
@@ -248,11 +263,12 @@ def publish_packages(endpoint: str,
         # Get the package name
         if package_name is None:
 
-            package_name = os.path.basename(upload_file).split('-')[0]
-            package_name = package_name.replace('_', '-')
+            package_name = get_package_name(upload_file)
 
         uploader.upload_package(upload_file, package_name, public,
                                 owner_full_control)
 
     # Update the index
-    uploader.upload_index(package_name)
+    uploader.upload_index(package_name,
+                          public=public,
+                          owner_full_control=owner_full_control)
